@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Niklas Knutsson   *
- *   nq@altern.org   *
+ *   Copyright (C) 2005-2007 by Niklas Knutsson                            *
+ *   nq@altern.org                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -29,6 +29,8 @@
 #include <qstylesheet.h>
 #include <ktextbrowser.h>
 #include <klistview.h>
+#include <kdialogbase.h>
+#include <kshortcut.h>
 
 class Prefix;
 class QalculatePrecisionDialog;
@@ -81,6 +83,8 @@ class KUniqueApplication;
 class KConfig;
 class QListViewItem;
 class QalculateRPNStackListView;
+class KGlobalAccel;
+class KKeyButton;
 
 void *view_proc(void *pipe);
 void *command_proc(void *pipe);
@@ -91,7 +95,23 @@ enum {
 };
 
 
+class ModeObject : public QObject {
+
+	Q_OBJECT
+	
+	public:
+
+		ModeObject(int mode);
+		int i_mode;
+
+	public slots:
+
+		void loadMode();
+		
+};
+
 class KQalculate : public KMainWindow {
+	
 	Q_OBJECT
 
 public:
@@ -111,7 +131,7 @@ public:
 	void result_prefix_changed(Prefix *prefix = NULL);
 	void expression_calculation_updated();
 	void expression_format_updated(bool recalculate = false);
-	void display_errors(QString *new_text = NULL);
+	void display_errors(QString *new_text = NULL, int *inhistory_index = NULL);
 	void update_status_text();
 	void display_function_hint(MathFunction *f, int arg_index = 1);
 	void clearresult();
@@ -198,7 +218,7 @@ public:
 	KMenuBar *menubar;
 	QPopupMenu *menu_functions, *menu_variables, *menu_units, *menu_to_unit, *menu_set_prefix, *menu_modes;
 	KToggleAction *ActionAbbreviateNames, *ActionReadPrecision, *ActionNonZeroDenominators, *ActionWarnAboutDenominatorsAssumedNonZero;
-	KToggleAction *ActionLimitImplicitMultiplication, *ActionRPNMode, *ActionRPNSyntax, *ActionIndicateInfiniteSeries;
+	KToggleAction *ActionLimitImplicitMultiplication, *ActionRPNMode, *ActionRPNSyntax, *ActionIndicateInfiniteSeries, *ActionNegativeExponents, *ActionSortMinusLast;
 	KToggleAction *ActionShowEndingZeroes, *ActionRoundHalfwayNumbersToEven, *ActionEnablePrefixes;
 	KToggleAction *ActionEnableUseOfAllPrefixes, *ActionEnableDenominatorPrefixes, *ActionPlaceUnitsSeparately;
 	KToggleAction *ActionEnableVariables, *ActionEnableFunctions, *ActionEnableUnits, *ActionEnableUnknowns;
@@ -206,9 +226,9 @@ public:
 	KRadioAction *ActionApproximationAlwaysExact, *ActionApproximationTryExact, *ActionApproximationApproximate;
 	KRadioAction *ActionAlgebraicModeSimplify, *ActionAlgebraicModeFactorize, *ActionAlgebraicModeNone;
 	KRadioAction *ActionFractionalDisplayDecimal, *ActionFractionalDisplayDecimalTryExact;
-	KRadioAction *ActionFractionalDisplayFraction, *ActionFractionalDisplayCombined, *ActionNumericalDisplayNormal;
-	KRadioAction *ActionNumericalDisplayScientific, *ActionNumericalDisplayPurelyScientific;
-	KRadioAction *ActionNumericalDisplaySimple, *ActionNumberBaseBinary, *ActionNumberBaseOctal, *ActionNumberBaseDecimal, *ActionNumberBaseHexadecimal, *ActionNumberBaseOther;
+	KRadioAction *ActionFractionalDisplayFraction, *ActionFractionalDisplayCombined;
+	KRadioAction *ActionNumericalDisplayEngineering, *ActionNumericalDisplayScientific, *ActionNumericalDisplayPurelyScientific, *ActionNumericalDisplaySimple, *ActionNumericalDisplayNormal;
+	KRadioAction *ActionNumberBaseBinary, *ActionNumberBaseOctal, *ActionNumberBaseDecimal, *ActionNumberBaseHexadecimal, *ActionNumberBaseOther;
 	KRadioAction *ActionNumberBaseSexagesimal, *ActionNumberBaseTimeFormat, *ActionNumberBaseRomanNumerals;
 	KRadioAction *ActionAssumptionTypeUnknown, *ActionAssumptionTypeNonMatrix, *ActionAssumptionTypeNumber, *ActionAssumptionTypeComplex, *ActionAssumptionTypeReal;
 	KRadioAction *ActionAssumptionTypeRational, *ActionAssumptionTypeInteger, *ActionAssumptionSignUnknown, *ActionAssumptionSignNonZero;
@@ -226,6 +246,12 @@ public:
 	KAction *ActionClearHistory;
 	KAction *ActionHideSystemTrayIcon, *ActionHide;
 	KAction *ActionClearStack, *ActionDeleteRegister, *ActionEditRegister;
+	KAction *ActionConfigureGlobalShortcuts;
+
+	KGlobalAccel *globalAccel;
+
+	QValueVector<int>::size_type current_history_index;
+	QValueVector<ModeObject*> mode_objects;
 
 public slots:
 
@@ -233,6 +259,9 @@ public slots:
 	void setResult2();
 	void execute_expression2();
 
+	void showHide();
+	void configureGlobalShortcuts();
+
 	void update_fmenu();
 	void update_vmenu();	
 	void update_umenus();
@@ -265,10 +294,13 @@ public slots:
 	void fractionalDisplayFraction();
 	void fractionalDisplayCombined();
 	void numericalDisplayNormal();
+	void numericalDisplayEngineering();
 	void numericalDisplayScientific();
 	void numericalDisplayPurelyScientific();
 	void numericalDisplaySimple();
 	void indicateInfiniteSeries(bool);
+	void sortMinusLast(bool);
+	void negativeExponents(bool);
 	void roundHalfwayNumbersToEven(bool);
 	void abbreviateNames(bool);
 	void updateBaseOther();
@@ -422,6 +454,10 @@ public slots:
 	void stackRegisterSet(QListViewItem*);
 	void popupStackMenu(KListView*, QListViewItem*, const QPoint&);
 	void registerSelected();
+	void editHistoryExpression();
+	void editHistoryParse();
+	void editHistoryResult();
+	void insertHistoryResultVariable();
 
 protected:
 
@@ -529,4 +565,29 @@ protected:
 
 };
 
+class QalculateModeDialog : public KDialogBase {
+	
+	Q_OBJECT
+	
+	protected:
+
+		KKeyButton *keyButton;
+		KComboBox *modeCombo;
+		
+	public:
+		
+		QalculateModeDialog(QWidget *parent);
+		virtual ~QalculateModeDialog();
+
+		QString modeName();
+		const KShortcut &modeShortcut();
+
+	public slots:
+		
+		void slotOk();
+		void updateShortcut(const KShortcut&);
+		void modeSelected(int);
+		
+};
+
 #endif // QALCULATE_KWINDOW_H
