@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Niklas Knutsson   *
+ *   Copyright (C) 2005-2007 by Niklas Knutsson   *
  *   nq@altern.org   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -86,6 +86,11 @@ bool show_keypad, show_history, show_stack;
 
 vector<mode_struct> modes;
 
+QValueVector<QString> inhistory;
+QValueVector<int> inhistory_type;
+QValueVector<int> inhistory_id;
+int inhistory_current_id;
+
 /*
 	save mode to file
 */
@@ -204,6 +209,8 @@ void load_preferences() {
 	save_mode_as(i18n("Default"));
 	size_t mode_index = 1;
 
+	inhistory_current_id = 0;
+
 	status_warning_color.setNamedColor("blue");
 	status_error_color.setNamedColor("red");
 	use_icon_buttons = 1;
@@ -237,7 +244,7 @@ void load_preferences() {
 		closedir(dir);
 	}
 
-	int version_numbers[] = {0, 9, 4};
+	int version_numbers[] = {0, 9, 6};
 	FILE *file = NULL;
 	filename += "qalculate-kde.cfg";
 	file = fopen(filename.c_str(), "r");
@@ -261,6 +268,8 @@ void load_preferences() {
 				v = svalue.toInt();
 				if(svar == "version") {
 					parse_qalculate_version(svalue.ascii(), version_numbers);
+				} else if(svar == "shortcut") {
+					if(mode_index != 1) modes[mode_index].shortcut.init(svalue);
 				} else if(svar == "save_mode_on_exit") {
 					save_mode_on_exit = v;
 				} else if(svar == "save_definitions_on_exit") {
@@ -514,11 +523,48 @@ void load_preferences() {
 					enable_plot_expression_completion = v;
 				} else if(svar == "expression_history") {
 					expression_history.push_back(svalue);
- 				} else if(svar == "history") {
+				} else if(svar == "history") {
 					initial_history = svalue;
 					if(version_numbers[1] < 9) {
 						initial_history.replace("<br><br>", "<br><hr>");
 					}
+					inhistory.push_back(initial_history);
+					inhistory_type.push_back(QALCULATE_HISTORY_OLD);
+				} else if(svar == "history_old") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_OLD);
+				} else if(svar == "history_expression") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_EXPRESSION);
+				} else if(svar == "history_transformation") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_TRANSFORMATION);
+				} else if(svar == "history_result") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_RESULT);
+				} else if(svar == "history_result_approximate") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_RESULT_APPROXIMATE);
+				} else if(svar == "history_parse") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_PARSE);
+				} else if(svar == "history_parse_approximate") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_PARSE_APPROXIMATE);
+				} else if(svar == "history_register_moved") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_REGISTER_MOVED);
+				} else if(svar == "history_warning") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_WARNING);
+				} else if(svar == "history_error") {
+					inhistory.push_back(svalue);
+					inhistory_type.push_back(QALCULATE_HISTORY_ERROR);
+				} else if(svar == "history_continued") {
+					if(inhistory.size() > 0) {
+						inhistory[inhistory.size() - 1] += "<br>";
+						inhistory[inhistory.size() - 1] += svalue;
+					}
  				} else if(svar == "recent_functions") {
 					int v_i = 0;
 					while(true) {
@@ -596,6 +642,13 @@ void load_preferences() {
 				}
 			}
 		}
+		inhistory_current_id = (int) inhistory.size();
+		int id = inhistory_current_id;
+		id--;
+		for(QValueVector<QString>::size_type i = 0; i < inhistory.size(); i++) {
+			inhistory_id.push_back(id);
+			id--;
+		}
 	} else {
 		first_time = true;
 	}
@@ -649,9 +702,74 @@ void save_preferences(bool mode) {
 	for (QStringList::Iterator it = ((QalculateExpressionEdit*) expressionWidget)->expression_history.begin(); it != ((QalculateExpressionEdit*) expressionWidget)->expression_history.end(); ++it) {
 		fprintf(file, "expression_history=%s\n", (*it).ascii());
 	}
-	QString str = mainWin->historyBrowser->text().section("<br>", 0, 50);
-	str.replace("\n", "<br>");
-	fprintf(file, "history=%s\n", str.ascii());
+	int lines = 50;
+	bool end_after_result = false;
+	bool doend = false;
+	for(QValueVector<QString>::size_type i = 0; i < inhistory.size() && !doend; i++) {
+		switch(inhistory_type[i]) {
+			case QALCULATE_HISTORY_EXPRESSION: {
+				fprintf(file, "history_expression=");
+				break;
+			}
+			case QALCULATE_HISTORY_TRANSFORMATION: {
+				fprintf(file, "history_transformation=");
+				break;
+			}
+			case QALCULATE_HISTORY_RESULT: {
+				fprintf(file, "history_result=");
+				lines--;
+				if(end_after_result) doend = true;
+				break;
+			}
+			case QALCULATE_HISTORY_RESULT_APPROXIMATE: {
+				fprintf(file, "history_result_approximate=");
+				lines--;
+				if(end_after_result) doend = true;
+				break;
+			}			
+			case QALCULATE_HISTORY_PARSE: {
+				fprintf(file, "history_parse=");
+				lines--;
+				if(lines < 0) end_after_result = true;
+				break;
+			}
+			case QALCULATE_HISTORY_PARSE_APPROXIMATE: {
+				fprintf(file, "history_parse_approximate=");
+				lines--;
+				if(lines < 0) end_after_result = true;
+				break;
+			}
+			case QALCULATE_HISTORY_REGISTER_MOVED: {
+				fprintf(file, "history_register_moved=");
+				break;
+			}
+			case QALCULATE_HISTORY_WARNING: {
+				fprintf(file, "history_warning=");
+				lines--;
+				break;
+			}
+			case QALCULATE_HISTORY_ERROR: {
+				fprintf(file, "history_error=");
+				lines--;
+				break;
+			}
+			case QALCULATE_HISTORY_OLD: {
+				fprintf(file, "history_old=");
+				lines--;
+				if(lines < 0) doend = true;
+				break;
+			}
+		}
+		if(inhistory[i].find("<br>") < 0) {
+			fprintf(file, "%s\n", inhistory[i].ascii());
+		} else {
+			QStringList slist = QStringList::split("<br>", inhistory[i], true);
+			for(QStringList::size_type i2 = 0; i2 < slist.size(); i2++) {
+				if(i2 == 0) fprintf(file, "%s\n", slist[i2].ascii());
+				else fprintf(file, "history_continued=%s\n", slist[i2].ascii());
+			}
+		}
+	}
 	fprintf(file, "recent_functions="); 
 	for(int i = (int) (recent_functions.size()) - 1; i >= 0; i--) {
 		fprintf(file, "%s", recent_functions[i]->referenceName().c_str()); 
@@ -677,6 +795,7 @@ void save_preferences(bool mode) {
 			fprintf(file, "\n[Mode]\n");
 		} else {
 			fprintf(file, "\n[Mode %s]\n", modes[i].name.ascii());
+			if(!modes[i].shortcut.toString().isNull()) fprintf(file, "shortcut=%s\n", modes[i].shortcut.toString().ascii());
 		}
 		fprintf(file, "min_deci=%i\n", modes[i].po.min_decimals);
 		fprintf(file, "use_min_deci=%i\n", modes[i].po.use_min_decimals);
