@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Niklas Knutsson   *
- *   nq@altern.org   *
+ *   Copyright (C) 2005 by Niklas Knutsson                                 *
+ *   nq@altern.org                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -28,6 +28,7 @@
 #undef getItem
 
 #include <qstring.h>
+#include <kshortcut.h>
 
 class QLabel;
 class QLineEdit;
@@ -57,6 +58,7 @@ struct mode_struct {
 	int precision;
 	QString name;
 	bool rpn_mode;
+	KShortcut shortcut;
 };
 
 void insert_text_in_expression(const QString  &str);
