@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Niklas Knutsson   *
- *   nq@altern.org   *
+ *   Copyright (C) 2005. 2007 by Niklas Knutsson                           *
+ *   nq@altern.org                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -387,9 +387,13 @@ void QalculateExpressionEdit::wrapSelection() {
 
 void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
 	if(e->key() == Key_Enter || e->key() == Key_Return) {
-		mainWin->execute();
-		e->accept();
-		return;
+		if(b_main) {
+			mainWin->execute();
+			e->accept();
+			return;
+		} else {
+			return KLineEdit::keyPressEvent(e);
+		}
 	}
 	if(e->state() & ControlButton && e->key() == Key_Asterisk) {
 		if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
@@ -405,6 +409,22 @@ void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
 		return;
 	}
 	switch(e->key()) {
+		case Key_Period: {
+			if(e->state() == Keypad) {
+				insert(CALCULATOR->getDecimalPoint());	
+				e->accept();
+				return;
+			}
+			break;
+		}
+		case Key_Comma: {
+			if(e->state() == Keypad) {
+				insert(CALCULATOR->getDecimalPoint());
+				e->accept();
+				return;
+			}
+			break;
+		}
 		case Key_AsciiCircum: {
 			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
 				mainWin->calculateRPN(OPERATION_RAISE);
@@ -626,6 +646,86 @@ bool QalculateExpressionEdit::eventFilter(QObject *o, QEvent *ev) {
 }
 
 
+void set_title(ExpressionItem *item, QString &title) {
+	if(item->type() != TYPE_VARIABLE || !item->title(false).empty()) {
+		title = item->title().c_str();
+	} else {
+		Variable *v = (Variable*) item;
+		if(is_answer_variable(v)) {
+			title = i18n("a previous result");
+		} else if(v->isKnown()) {
+			if(((KnownVariable*) v)->isExpression()) {
+				title = CALCULATOR->localizeExpression(((KnownVariable*) v)->expression()).c_str();
+			} else {
+				if(((KnownVariable*) v)->get().isMatrix()) {
+					title = i18n("matrix");
+				} else if(((KnownVariable*) v)->get().isVector()) {
+					title = i18n("vector");
+				} else {
+					title = CALCULATOR->printMathStructureTimeOut(((KnownVariable*) v)->get(), 30).c_str();
+				}
+			}
+		} else {
+			if(((UnknownVariable*) v)->assumptions()) {
+				switch(((UnknownVariable*) v)->assumptions()->sign()) {
+					case ASSUMPTION_SIGN_POSITIVE: {
+						title = i18n("positive");
+						break;
+					}
+					case ASSUMPTION_SIGN_NONPOSITIVE: {
+						title = i18n("non-positive");
+						break;
+					}
+					case ASSUMPTION_SIGN_NEGATIVE: {
+						title = i18n("negative");
+						break;
+					}
+					case ASSUMPTION_SIGN_NONNEGATIVE: {
+						title = i18n("non-negative");
+						break;
+					}
+					case ASSUMPTION_SIGN_NONZERO: {
+						title = i18n("non-zero");
+						break;
+					}
+					default: {}
+				}
+				if(!title.isEmpty() && !((UnknownVariable*) v)->assumptions()->type() == ASSUMPTION_TYPE_NONE) title += " ";
+				switch(((UnknownVariable*) v)->assumptions()->type()) {
+					case ASSUMPTION_TYPE_INTEGER: {
+						title += i18n("integer");
+						break;
+					}
+					case ASSUMPTION_TYPE_RATIONAL: {
+						title += i18n("rational");
+						break;
+					}
+					case ASSUMPTION_TYPE_REAL: {
+						title += i18n("real");
+						break;
+					}
+					case ASSUMPTION_TYPE_COMPLEX: {
+						title += i18n("complex");
+						break;
+					}
+					case ASSUMPTION_TYPE_NUMBER: {
+						title += i18n("number");
+						break;
+					}
+					case ASSUMPTION_TYPE_NONMATRIX: {
+						title += i18n("(not matrix)");
+						break;
+					}
+					default: {}
+				}
+				if(title.isEmpty()) title = i18n("unknown");
+			} else {
+				title = i18n("default assumptions");
+			}
+		}
+	}
+}
+
 string sub_suffix(const ExpressionName *ename) {
 	size_t i = ename->name.rfind('_');
 	bool b = i == string::npos || i == ename->name.length() - 1 || i == 0;
@@ -687,7 +787,7 @@ QalculateExpressionEditListBoxItem::QalculateExpressionEditListBoxItem(Expressio
 	item = eitem;
 	italic_index = -1;
 	setText(makeListName(item, listBox(), &italic_index, &rich_text));
-	title = item->title().c_str();
+	set_title(item, title);
 }
 QalculateExpressionEditListBoxItem::~QalculateExpressionEditListBoxItem() {
 }
@@ -784,7 +884,7 @@ bool QalculateExpressionEditListBoxItem::reuse(ExpressionItem *newItem) {
 	item = newItem;
 	italic_index = -1;
 	setText(makeListName(item, listBox(), &italic_index, &rich_text));
-	title = item->title().c_str();
+	set_title(item, title);
 	return true;
 }
 
