@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Niklas Knutsson   *
- *   nq@altern.org   *
+ *   Copyright (C) 2005 by Niklas Knutsson                                 *
+ *   nq@altern.org                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -74,7 +74,7 @@ QalculateImportCSVDialog::QalculateImportCSVDialog(QWidget *parent, const char *
 	hbox = new QHBoxLayout(0, 0, spacingHint());
 	grid->addMultiCellLayout(hbox, 6, 6, 0, 1);
 	hbox->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum));
-	includesHeadingsButton = new QCheckBox(i18n("Include headings"), mainWidget());
+	includesHeadingsButton = new QCheckBox(i18n("Includes headings"), mainWidget());
 	includesHeadingsButton->setChecked(true);
 	hbox->addWidget(includesHeadingsButton);
 	grid->addWidget(new QLabel(i18n("Delimiter:"), mainWidget()), 7, 0);
